"""
Django settings for hologger project.

Generated by 'django-admin startproject' using Django 2.2.6.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'f@*@ft3on$j!9+yk45t#q%n37oo=7t$1#6ju9)sctw)&*43eu_'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'rest_framework',
    'log_test.apps.LogTestConfig',
    'hologger',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'hologger.middleware.HoLoggingMiddleware',
]

ROOT_URLCONF = 'hologger.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'hologger.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': 'admin',
        'USER': 'admin',
        'PASSWORD': 'trustus12#',
        'HOST': 'os-qa-db.mypoing.com',
        'PORT': '3306',
        'ATOMIC_REQUESTS': True,
        'CONN_MAX_AGE': 60,
        'OPTIONS': {'charset': 'utf8mb4', 'init_command': "SET sql_mode='STRICT_TRANS_TABLES'"}
    },
    'ho_logger': {
        'ENGINE': 'django.db.backends.mysql',
        'NAME': 'logging',
        'USER': 'admin',
        'PASSWORD': 'trustus12#',
        'HOST': 'os-qa-db.mypoing.com',
        'PORT': '3306',
        'ATOMIC_REQUESTS': True,
        'CONN_MAX_AGE': 60,
        'OPTIONS': {'charset': 'utf8mb4', 'init_command': "SET sql_mode='STRICT_TRANS_TABLES'"}
    }
}

DATABASE_ROUTERS = ['hologger.router.HoLoggingRouter']


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = '/static/'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class' : 'logging.StreamHandler'
        },
        'hologger': {
            'level': 'DEBUG',
            'class': 'hologger.handlers.HoLogHandler',
        },
        'hologger_sql':{
            'level' : 'DEBUG',
            'class' : 'hologger.handlers.HoLogSqlHandler'
        }
    },
    'loggers': {
        'hologger': {
            'handlers': ['hologger'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'django.db.backends':{
            'handlers': ['hologger_sql'],
            'level': 'DEBUG'
        },
        'common':{
            'handlers': ['console', 'hologger_sql'],
            'level': 'INFO'
        }
    },
}


import json
from datetime import datetime
import threading

from django.http import Http404
from django.urls import resolve
from django.utils.deprecation import MiddlewareMixin
import os


class HoLoggingMiddleware(MiddlewareMixin):

    thread_local = threading.local()

    def process_request(self, request):
        self.thread_local.pid = os.getpid()
        self.thread_local.logs = self.thread_local.end_at = self.thread_local.response = None
        self.thread_local.request = request
        self.thread_local.current_user = request.user
        self.thread_local.body = {'query_params': request.GET.dict(), 'body': {}, 'files': {}}
        if len(request.body) > 0 and request.META['CONTENT_TYPE'] == 'application/json':
            self.thread_local.body['body'] = json.loads(request.body)

        self.thread_local.start_at = datetime.now().isoformat()

        try:
            self.thread_local.application = resolve(request.path).func.__module__.split('.')[0]
        except Http404:
            self.thread_local.application = ''

    def process_exception(self, request, exception):
        self.process_request(request)

    def process_response(self, request, response):
        self.thread_local.end_at = datetime.now().isoformat()
        self.thread_local.response = response
        return response



from django.db import models
from pytz import timezone
from django.conf import settings


class ApiResponse(models.Model):
    ho_logger = True

    status = models.CharField(max_length=30)
    host = models.CharField(max_length=150)
    uri = models.CharField(max_length=200)
    method = models.CharField(max_length=10)
    remote_addr = models.CharField(max_length=20)
    header = models.TextField(default="", blank=True)
    body = models.TextField(default="", blank=True)
    params = models.Field
    response = models.TextField(default="", blank=True, null=True)
    pid = models.CharField(max_length=10)
    logs = models.TextField(default="", blank=True, null=True)
    duration = models.FloatField(null=True)

    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        db_table = "api_response"
        managed = True
        indexes = [
            models.Index(fields=['status', ]),
        ]

    @property
    def created_at_korean_time(self):
        korean_timezone = timezone(settings.TIME_ZONE)
        return self.created_at.astimezone(korean_timezone)
